 /**************************************************************************
 *   Copyright (C) 2005 by Vlad Svetlichny                                 *
 *   vladsv@inbox.ru                                                       *
 *   Distributed under GPL                                                 *
 ***************************************************************************/
 #include "prefs.h"
 #include <qpushbutton.h>
 #include <qcheckbox.h>
 #include <qcolordialog.h> 
 #include <qfontdialog.h>
 #include <qpixmap.h>
 #include <qlabel.h>
 #include <qcombobox.h>
 #include <qlayout.h>
 #include <qfiledialog.h>
 #include <qmessagebox.h>
 #include "settings.h"
 
prefs::prefs(QWidget *parent, Settings *set)
 : QTabDialog(parent)
{
    setCaption(tr("Preferences"));
    setIcon(QPixmap::fromMimeSource("configure.png"));
//     QMessageBox::information(this,"Debug","Before all");
    paper=set->paper;
    defaultCode=set->defaultCode;
    autoCodeRus=set->autoCodeRus;
    autoCodeUa=set->autoCodeUa;
    autoCodeFinnish=set->autoCodeFinnish;
    autoCodePortuguese=set->autoCodePortuguese;
    autoCodeSlovak=set->autoCodeSlovak;
    autoCodeSlovenian=set->autoCodeSlovenian;
    autoCodeTurkish=set->autoCodeTurkish;
    autoCodeSpanish=set->autoCodeSpanish;
    autoCodePolish=set->autoCodePolish;
    autoCodeLatvian=set->autoCodeLatvian;
    autoCodeGerman=set->autoCodeGerman;
    autoCodeSerbian=set->autoCodeSerbian;
    confirmOnExit=set->confirmOnExit;
    confirmOnBmDel=set->confirmOnBmDel;
    confirmOnCloseAll=set->confirmOnCloseAll;
    sett=set;
    setupTabLnf();
    setupTabEnc();
    setupTabConfirm();
    setupTabMisc();
    setOkButton(tr("OK"));
    setCancelButton(tr("Cancel"));
    f_back=false;
    f_font=false;
    f_fore=false;
    
    connect(this,SIGNAL(applyButtonPressed()),SLOT(apply()));
}


void prefs::setupTabLnf()
{
    QWidget *tab=new QWidget(this);
    QPushButton *clrBackBtn= new QPushButton(tr("Change background color"),tab);
    QPushButton *clrForeBtn= new QPushButton(tr("Change foreground color"),tab);
    QPushButton *fontBtn= new QPushButton(tr("Change font"),tab);
    pprChk=new QCheckBox(tr("Use image for background"),tab);
    pprBtn= new QPushButton(tr("Set image..."),tab);
    
    pprChk->setChecked(sett->usePaper);
    pprBtn->setEnabled(sett->usePaper);
    
    QVBoxLayout *vbox=new QVBoxLayout(tab);
    vbox->addWidget(clrBackBtn);
    vbox->addWidget(clrForeBtn);
    vbox->addWidget(fontBtn);
    QHBoxLayout *hbox=new QHBoxLayout;
    hbox->addWidget(pprChk);
    hbox->addWidget(pprBtn);
    vbox->addLayout(hbox);
    pprEdit=new QLineEdit(paper,tab);
    pprEdit->setEnabled(sett->usePaper);
    vbox->addWidget(pprEdit);
    
    vbox->setMargin(5);
    vbox->setSpacing(5);
    hbox->setSpacing(5);
    
    connect(clrBackBtn,SIGNAL(clicked()),SLOT(changeBackground()));
    connect(clrForeBtn,SIGNAL(clicked()),SLOT(changeForeground()));
    connect(fontBtn,SIGNAL(clicked()),SLOT(changeFont()));
    connect(pprChk,SIGNAL(toggled(bool)),SLOT(pprChecked(bool)));
    connect(pprBtn,SIGNAL(clicked()),SLOT(changePaper()));
    
    addTab(tab, tr("Look && Feel"));
}

void prefs::setupTabEnc()
{
    QWidget *tab=new QWidget(this);
    QLabel *lbl=new QLabel(tr("Select default encoding"),tab);
    selCdpg=new QComboBox(tab);
    
    
    selCdpg->insertItem("Latin1");
    selCdpg->insertItem("Big5");
    selCdpg->insertItem("Big5-HKSCS");
    selCdpg->insertItem("eucJP");
    selCdpg->insertItem("eucKR");
    selCdpg->insertItem("GB2312");
    selCdpg->insertItem("GBK");
    selCdpg->insertItem("GB18030");
    selCdpg->insertItem("JIS7");
    selCdpg->insertItem("Shift-JIS");
    selCdpg->insertItem("TSCII ");
    selCdpg->insertItem("utf8");
    selCdpg->insertItem("utf16");
    selCdpg->insertItem("KOI8-R");
    selCdpg->insertItem("KOI8-U");
    selCdpg->insertItem("ISO8859-1");
    selCdpg->insertItem("ISO8859-2");
    selCdpg->insertItem("ISO8859-3");
    selCdpg->insertItem("ISO8859-4");
    selCdpg->insertItem("ISO8859-5");
    selCdpg->insertItem("ISO8859-6");
    selCdpg->insertItem("ISO8859-7");
    selCdpg->insertItem("ISO8859-8");
    selCdpg->insertItem("ISO8859-8-i");
    selCdpg->insertItem("ISO8859-9");
    selCdpg->insertItem("ISO8859-10");
    selCdpg->insertItem("ISO8859-13");
    selCdpg->insertItem("ISO8859-14");
    selCdpg->insertItem("ISO8859-15");
    selCdpg->insertItem("IBM 850");
    selCdpg->insertItem("IBM 866");
    selCdpg->insertItem("CP874");
    selCdpg->insertItem("CP1250");
    selCdpg->insertItem("CP1251");
    selCdpg->insertItem("CP1252");
    selCdpg->insertItem("CP1253");
    selCdpg->insertItem("CP1254");
    selCdpg->insertItem("CP1255");
    selCdpg->insertItem("CP1256");
    selCdpg->insertItem("CP1257");
    selCdpg->insertItem("CP1258");
    selCdpg->insertItem("Apple Roman");
    selCdpg->insertItem("TIS-620");

    selCdpg->setCurrentText(defaultCode);

    ukrChk=new QCheckBox(tr("Autodetect for Ukrainian"),tab);
        ukrChk->setChecked(autoCodeUa);
    rusChk=new QCheckBox(tr("Autodetect for Russian"  ),tab);
        rusChk->setChecked(autoCodeRus);

    finnishChk=new QCheckBox(tr("Autodetect for Finnish"),tab);
    finnishChk->setChecked(autoCodeFinnish);
    portugueseChk=new QCheckBox(tr("Autodetect for Portuguese"),tab);
    portugueseChk->setChecked(autoCodePortuguese);
    slovakChk=new QCheckBox(tr("Autodetect for Slovak"),tab);
    slovakChk->setChecked(autoCodeSlovak);
    slovenianChk=new QCheckBox(tr("Autodetect for Slovenian"),tab);
    slovenianChk->setChecked(autoCodeSlovenian);
    turkishChk=new QCheckBox(tr("Autodetect for Turkish"),tab);
    turkishChk->setChecked(autoCodeTurkish);
    spanishChk=new QCheckBox(tr("Autodetect for Spanish"),tab);
    spanishChk->setChecked(autoCodeSpanish);
    polishChk=new QCheckBox(tr("Autodetect for Polish"),tab);
    polishChk->setChecked(autoCodePolish);
    latvianChk=new QCheckBox(tr("Autodetect for Latvian"),tab);
    latvianChk->setChecked(autoCodeLatvian);
    germanChk=new QCheckBox(tr("Autodetect for German"),tab);
    germanChk->setChecked(autoCodeGerman);
    serbianChk=new QCheckBox(tr("Autodetect for Serbian"),tab);
    serbianChk->setChecked(autoCodeSerbian);
        
        
    QVBoxLayout *vbox=new QVBoxLayout(tab);
    QHBoxLayout *hbox=new QHBoxLayout;
    hbox->addWidget(lbl);
    hbox->addWidget(selCdpg);
    vbox->addLayout(hbox);
    vbox->addWidget(ukrChk);
    vbox->addWidget(rusChk);
    vbox->addWidget(finnishChk);
    vbox->addWidget(portugueseChk);
    vbox->addWidget(slovakChk);
    vbox->addWidget(slovenianChk);
    vbox->addWidget(turkishChk);
    vbox->addWidget(spanishChk);
    vbox->addWidget(polishChk);
    vbox->addWidget(latvianChk);
    vbox->addWidget(germanChk);
    vbox->addWidget(serbianChk);
    
    vbox->setMargin(5);
    vbox->setSpacing(5);
    hbox->setSpacing(5);
    
    addTab(tab, tr("Encodings"));
}

void prefs::setupTabConfirm()
{
    QWidget *tab=new QWidget(this);
    
    QVBoxLayout *vbox=new QVBoxLayout(tab);
    vbox->setAutoAdd(TRUE);
    
    onExitChk=new QCheckBox(tr("Exit the program"),tab);
        onExitChk->setChecked(confirmOnExit);
    onCloseAllChk=new QCheckBox(tr("Close all pages"),tab);
        onCloseAllChk->setChecked(confirmOnCloseAll);
    onBmDelChk=new QCheckBox(tr("Delete a bookmark"),tab);
        onBmDelChk->setChecked(confirmOnBmDel);
    
    vbox->setMargin(5);
    vbox->setSpacing(5);
    
    addTab(tab, tr("Confirmations"));
}

void prefs::setupTabMisc()
{
    QWidget *tab=new QWidget(this);
    
    QVBoxLayout *vbox=new QVBoxLayout(tab);
    
    vbox->addWidget(new QLabel(tr("Autoscrolling speed (msec):"),tab));
    
    QHBoxLayout *hbox=new QHBoxLayout(vbox);
    hbox->addWidget(sldrScrollSpeed=new
            QSlider(10,500,10,sett->scroll,Qt::Horizontal,tab));
    hbox->addWidget(spinScrollSpeed=new
            QSpinBox(10,500,10,tab));
            
    vbox->addWidget(new QLabel(tr("Sidebar width (pixels):"),tab));
    vbox->addWidget(spinSbWidth=new
            QSpinBox(100,700,10,tab));
            
    spinScrollSpeed->setValue(sett->scroll);
    spinSbWidth->setValue(sett->sbWidth);
    vbox->setMargin(5);
    vbox->setSpacing(5);
    hbox->setSpacing(5);
    
    connect(sldrScrollSpeed,SIGNAL(valueChanged(int)),
            spinScrollSpeed,SLOT(setValue(int)));
    
    addTab(tab, tr("Misc"));
}

void prefs::changeBackground()
{
    back=QColorDialog::getColor(sett->background,this);
    f_back=back.isValid();
}

void prefs::changeForeground()
{
    fore=QColorDialog::getColor(sett->foreground,this);
    f_fore=fore.isValid();
}

void prefs::changeFont()
{
    font=QFontDialog::getFont(&f_font,sett->font,this);
}

void prefs::changePaper()
{
    pprEdit->setText(QFileDialog::getOpenFileName( QString::null,
    "Image files (*.png *.PNG *.jpg *.JPG *.bmp *.BMP)",
    this,
    tr("Open a pixmap"),
    tr("Choose a file")));
}

void prefs::pprChecked(bool check)
{
    pprBtn->setEnabled(check);
    pprEdit->setEnabled(check);
}

void prefs::apply()
{
    if (f_back) sett->background=back;
    if (f_fore) sett->foreground=fore;
    if (f_font) sett->font=font;
    sett->paper=pprEdit->text();
    sett->autoCodeRus=rusChk->isChecked();
    sett->autoCodeUa =ukrChk->isChecked();
    sett->autoCodeFinnish =finnishChk->isChecked();
    sett->autoCodePortuguese =portugueseChk->isChecked();
    sett->autoCodeSlovak =slovakChk->isChecked();
    sett->autoCodeSlovenian =slovenianChk->isChecked();
    sett->autoCodeTurkish =turkishChk->isChecked();
    sett->autoCodeSpanish =spanishChk->isChecked();
    sett->autoCodePolish =polishChk->isChecked();
    sett->autoCodeLatvian =latvianChk->isChecked();
    sett->autoCodeGerman =germanChk->isChecked();
    sett->autoCodeSerbian =serbianChk->isChecked();
    sett->usePaper=pprChk->isChecked();
    sett->defaultCode=selCdpg->currentText();
    sett->confirmOnBmDel=onBmDelChk->isChecked();
    sett->confirmOnCloseAll=onCloseAllChk->isChecked();
    sett->confirmOnExit=onExitChk->isChecked();
    sett->scroll=spinScrollSpeed->value();
    sett->sbWidth=spinSbWidth->value();
    sett->saveSettings();
}

prefs::~prefs()
{
}
