 /**************************************************************************
 *   Copyright (C) 2005 by Vlad Svetlichny                                 *
 *   vladsv@inbox.ru                                                       *
 *   Distributed under GPL                                                 *
 ***************************************************************************/
 #include "tab.h"
 #include "settings.h"
 
 #include <qfileinfo.h>
 #include <qtextedit.h>
 #include <qtextstream.h>
 #include <qfile.h>
 #include <qtextcodec.h>
 #include <qstylesheet.h>
 #include <qprocess.h>
 #include <qmessagebox.h>
 #include <qbrush.h>
 #include <qpixmap.h>

tab::tab(QWidget *parent, Settings *set, QString fName, int position, QString enc)
 : QTextEdit(parent)
{
    viewport()->setAcceptDrops(false);
    sset=set;
    refont();
    charset=set->defaultCode;
    setReadOnly(TRUE);
    realFileName=fileName=fName;
    autoCodeRus=set->autoCodeRus;
    autoCodeUa =set->autoCodeUa;
    autoCodeFinnish=set->autoCodeFinnish;
    autoCodePortuguese=set->autoCodePortuguese;
    autoCodeSlovak=set->autoCodeSlovak;
    autoCodeSlovenian=set->autoCodeSlovenian;
    autoCodeTurkish=set->autoCodeTurkish;
    autoCodeSpanish=set->autoCodeSpanish;
    autoCodePolish=set->autoCodePolish;
    autoCodeLatvian=set->autoCodeLatvian;
    autoCodeGerman=set->autoCodeGerman;
    autoCodeSerbian=set->autoCodeSerbian;
    loadFile(TRUE,position,enc);
    recolor();
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
    
    connect(this,SIGNAL(contentsMoving(int,int)),SLOT(changeXY(int,int)));
}

void tab::recolor()
{
    setColor(sset->foreground);
    setPaletteForegroundColor(sset->foreground);
    if ((sset->usePaper) && (sset->paper!=""))
    {
        QBrush paper;
	paper.setPixmap(QPixmap(sset->paper));
	if (paper.pixmap()) setPaper( paper );
    }
    else
    setPaper(QBrush(sset->background));
}

void tab::refont()
{
    setFont(sset->font);
}

void tab::loadFile(bool doAuto, int position, QString enc)
{   
    bool richtext=FALSE;
    QFileInfo fi(realFileName);
    QString ext=fi.extension();
    if ((ext=="DOC")||(ext=="doc")) {realFileName=convertDOC();enc="UTF-8";
                                     richtext=TRUE;}
    else if ((ext=="RTF")||(ext=="rtf")) {realFileName=convertRTF();
                                          richtext=TRUE;}
    else if ((ext=="fb2")||(ext=="FB2")) {realFileName=convertFB2();
                                          richtext=TRUE;}
    else if ((ext=="SXW")||(ext=="sxw")) {realFileName=convertOO();
                                          richtext=TRUE;}
    else if ((ext=="KWD")||(ext=="kwd")) {realFileName=convertKWD();
                                          richtext=TRUE;}
    
    if (!(enc==QString::null)) 
    {
        charset=enc;
        doAuto=FALSE;
    }
    QFile f2(realFileName);
    QTextStream ts;
    ts.setDevice(&f2);
    
    if (doAuto){
        enc="";
        if (autoCodeUa) {
            enc=detectUkrainian(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeRus && (enc=="")) {
            enc=detectRussian(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeSpanish && (enc=="")) {
            enc=detectSpanish(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeFinnish && (enc=="")) {
            enc=detectFinnish(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodePortuguese && (enc=="")) {
            enc=detectPortuguese(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeSlovak && (enc=="")) {
            enc=detectSlovak(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeSlovenian && (enc=="")) {
            enc=detectSlovenian(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeTurkish && (enc=="")) {
            enc=detectSpanish(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodePolish && (enc=="")) {
            enc=detectPolish(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeLatvian && (enc=="")) {
            enc=detectLatvian(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeGerman && (enc=="")) {
            enc=detectGerman(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
        if (autoCodeSerbian && (enc=="")) {
            enc=detectSerbian(&f2);
            if (enc!="") {
            ts.setCodec(QTextCodec::codecForName(enc));
            charset=enc;}
            else ts.setCodec(QTextCodec::codecForName(charset));
        }
    }
    else ts.setCodec(QTextCodec::codecForName(charset));
    
    f2.close();
    f2.open( IO_ReadOnly );
    
    setText(ts.read());
    setCursorPosition(position,0);
    f2.close();
}

void tab::recode(QString code)
{
    int fposition=paragraphAt(XY);
    charset=code;
    loadFile(FALSE,fposition);
}

QString tab::detectRussian(QFile *f)
{
   char *a[15] = {
                  " на ",
                  " не ",
                  " без ",
                  " тут ",
                  " же ",
                  " до ",
                  " еще ",
                  " вы ",
                  " мой ",
                  " его ",
                  " ее ",
                  " вас ",
                  " нас ",
                  " них ",
                  " их "
                 };
              
   QString codecs[5] = {
                  "IBM 866",
                  "ISO8859-5",
                  "KOI8-R",
                  "CP1251",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<5; i++)
  {   
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<15; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
              return codecs[i];
      }
      f->close();
  }
  
  return QString(""); 
}


QString tab::detectUkrainian (QFile *f)
{
   char *a[15] = {
                  " ви ",
                  " ще ",
                  " які",
                  " або ",
                  " ніж ",
                  " він ",
                  " що ",  
                  " його ",
                  " із ",
                  " між ",
                  " ім'я ",
                  " під ",
                  " цю ",
                  " речі ",
                  " їх "
                  };
  QString codecs[3] = {
                  "CP1251",
                  "KOI8-U",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<3; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<15; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString(""); 
  
}

/*
Codecs:
finnish - ISO-8859-15
portuguese - ISO-8859-1
slovak - ISO-8859-2
slovenian - ISO-8859-2
turkish - ISO-8859-9
spanish - ISO-8859-1
polish - ISO-8859-2
latvian - ISO-8859-13
german - ISO-8859-15
serbian - ISO-8859-2 */

QString tab::detectFinnish (QFile *f)
{
   char *a[8] = {
                 " eri ",
                 " sen ",
                 " ovat ",
                 " voi ",
                 " mutta ",
                 " joka ",
                 " jotta ",
                 " sen "
                 };
   QString codecs[2] = {
                  "ISO8859-15",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<8; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectPortuguese (QFile *f)
{
   char *a[4] = {
                 " por ",
                 " pelo ",
                 " um ",
                 " uma "
                };
   QString codecs[2] = {
                  "ISO8859-1",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<4; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectSlovak (QFile *f)
{
   char *a[6] = {
                 " tomto ",
                 " sa ",
                 " jeho ",
                 " pre ",
                 " bolo ",
                 " ste "
                };
   QString codecs[2] = {
                  "ISO8859-2",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<6; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectSlovenian (QFile *f)
{
   char *a[6] = {
                 " sta ",
                 " ki ",
                 " kot ",
                 " ter ",
                 " tudi ",
                 " vec "
                 };
   QString codecs[2] = {
                  "ISO8859-2",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<6; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectTurkish (QFile *f)
{
   char *a[4] = {
                 " bir ",
                 " bu ",
                 " ile ",
                 " olan "
                };
   QString codecs[2] = {
                  "ISO8859-9",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<4; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectSpanish (QFile *f)
{
   char *a[6] = {
                 " los ",
                 " las ",
                 " sobre ",
                 " dar ",
                 " del ",
                 " por "
                 };
   QString codecs[2] = {
                  "ISO8859-1",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<6; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectPolish (QFile *f)
{
   char *a[7] = {
                 " nie ",
                 " gdzie ",
                 " sie ",
                 " wam ",
                 " jest ",
                 " mozna ",
                 " moze "
                 };
   QString codecs[2] = {
                  "ISO8859-2",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<7; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectLatvian (QFile *f)
{
   char *a[6] = {
                 " lai ",
                 " visu ",
                 " visi ",
                 " arī ",
                 " ir ",
                 " vai "
                 };
   QString codecs[2] = {
                  "ISO8859-13",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<6; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectGerman (QFile *f)
{
   char *a[8] = {
                 " der ",
                 " das ",
                 " ich ",
                 " ist ",
                 " uns ",
                 " dem ",
                 " und ",
                 " auf "
                 };
   QString codecs[2] = {
                  "ISO8859-15",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<8; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

QString tab::detectSerbian (QFile *f)
{
   char *a[7] = {
                 " kako ",
                 " koji ",
                 " druge ",
                 " koja ",
                 " skup ",
                 " ako ",
                 " drugu "
                 };
   QString codecs[2] = {
                  "ISO8859-2",
                  "UTF-8"
                  };
  QString s;
  for (int i=0; i<2; i++)
  {
      QTextStream ts;
      ts.setDevice(f);
      f->open(IO_ReadOnly);
      ts.setCodec(QTextCodec::codecForName(codecs[i]));
      s=ts.read();
      for (int j=0; j<7; j++)
      {
          if (s.contains(QString::fromUtf8(a[j])))
          return codecs[i];
      }
      f->close();
  }
  
  return QString("");
}

void tab::changeXY(int x, int y)
{
  XY.setX(x);
  XY.setY(y);
}

void tab::savePosition()
{
  save_para=paragraphAt(XY);
}

void tab::loadPosition()
{
  setCursorPosition(save_para,0);
}

QString tab::convertDOC()
{
    QProcess *p = new QProcess( QString("wvHtml"), this );
    p->addArgument( realFileName );

    QDir tmp = QDir::homeDirPath() + "/.kniga/tmp/doc";
    p->addArgument( "--targetdir=" + tmp.path() );
    p->addArgument( "doc.html" );

    if ( !tmp.exists() )
        {
            tmp.mkdir( QDir::homeDirPath() + "/.kniga");
            tmp.mkdir( tmp.path() );
        }
		
	if ( !p->start() ) {
		QMessageBox::critical( this,
                tr("Error"),
                tr("WV (http://sourceforge.net/projects/wvware)\nisn't installed"));
	}
	while (p->isRunning());
	delete p;
	return tmp.path() + "/doc.html"; 
}

QString tab::convertRTF()
{
    return realFileName;
}


//TODO  KWD Opening

QString tab::convertKWD()
{
  QProcess *p = new QProcess( QString("zip"), this );
  p->addArgument( realFileName );

  QDir tmp = QDir::homeDirPath() + "/.kniga/tmp/kwd";
  p->addArgument( "-d " + tmp.path() );
  p->addArgument( tmp.path() );
  
  return realFileName;
}

QString tab::convertFB2()
{
    return realFileName;
}

QString tab::convertOO()
{
    return realFileName;
}

QString tab::convertHTML()
{
    return realFileName;
}